<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 28/10/19
 * Time: 16.55
 */

namespace FiloBlu\Rma\Model;
use FiloBlu\Rma\Api\ChangeSizeInterface;
use FiloBlu\Rma\Helper\ChangeSizeOrder;
use Magento\Sales\Model\Order\ItemRepository;
use Magento\Catalog\Model\ProductRepository;
use Magento\ConfigurableProduct\Api\LinkManagementInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;
use Magento\CatalogInventory\Api\StockStateInterface;
use Magento\Customer\Model\Session;
use Psr\Log\LoggerInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Catalog\Model\ResourceModel\Product;
use FiloBlu\Rma\Helper\RmaHelper;
use Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\Configurable;

class ChangeSize implements ChangeSizeInterface
{

    const STOCK_MIN_QTY = 'filoblu_disablerma_section/rma_group/rma_minimum_qty';

    /**
     * @var ChangeSizeOrder
     */
    protected $changeSizeOrder;

    /**
     * @var ItemRepository
     */
    protected $itemRepository;

    /**
     * @var ProductRepository
     */
    protected $productRepository;

    /**
     * @var LinkManagementInterface
     */
    protected $linkManagment;

    /**
     * @var mixed
     */
    protected $storeManager;

    /**
     * @var JsonFactory
     */
    protected $jsonFactory;
    /**
     * @var CollectionFactory
     */
    protected $collectionFactory;

    /**
     * @var StockStateInterface
     */
    protected $stockItem;

    /**
     * @var Session
     */
    protected $customerSession;

    /**
     * @var Logger
     */
    protected $logger;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var \Magento\Catalog\Model\ResourceModel\Product
     */
    protected $resourceModel;

    /**
     * @var Configurable
     */
    protected $configurableResourceModel;

    /**
     * @var RmaHelper
     */
    protected $rmaHelper;


    /**
     * ChangeSize constructor.
     * @param ChangeSizeOrder $changeSizeOrder
     * @param ItemRepository $itemRepository
     * @param ProductRepository $productRepository
     * @param LinkManagementInterface $linkManagement
     * @param StoreManagerInterface $storeManager
     * @param JsonFactory $jsonFactory
     * @param CollectionFactory $collectionFactory
     * @param StockStateInterface $stockItem
     * @param Session $customerSession
     * @param LoggerInterface $logger
     * @param ScopeConfigInterface $scopeConfig
     * @param Product $resourceModel
     * @param Configurable $configurableResourceModel
     * @param RmaHelper $rmaHelper
     */
    public function __construct(
        ChangeSizeOrder $changeSizeOrder,
        ItemRepository $itemRepository,
        ProductRepository $productRepository,
        LinkManagementInterface $linkManagement,
        StoreManagerInterface $storeManager,
        JsonFactory $jsonFactory,
        CollectionFactory $collectionFactory,
        StockStateInterface $stockItem,
        Session $customerSession,
        LoggerInterface $logger,
        ScopeConfigInterface $scopeConfig,
        Product $resourceModel,
        Configurable $configurableResourceModel,
        RmaHelper $rmaHelper
    )
    {
        $this->changeSizeOrder =$changeSizeOrder;
        $this->itemRepository =$itemRepository;
        $this->productRepository = $productRepository;
        $this->linkManagment = $linkManagement;
        $this->storeManager = $storeManager ?: ObjectManager::getInstance()->get(StoreManagerInterface::class);
        $this->jsonFactory = $jsonFactory;
        $this->collectionFactory = $collectionFactory;
        $this->stockItem = $stockItem;
        $this->customerSession =$customerSession;
        $this->logger = $logger;
        $this->scopeConfig = $scopeConfig;
        $this->resourceModel = $resourceModel;
        $this->configurableResourceModel = $configurableResourceModel;
        $this->rmaHelper = $rmaHelper;
    }


    /**
     * @param int $itemId
     * @param int $qty
     * @param string $resolution
     * @param int $reason
     * @return array
     */
    public function getSizes(int $itemId,int $qty,string $resolution, int $reason)
    {
        //check if reason is allowed for ChangeSize Order
        if($this->rmaHelper->checkIfAllowedMagentoChangeSizeReason($reason)) {
            try {
                $size = [];
                if ($resolution == $this->changeSizeOrder::REASON_CHANGE_SIZE_LABEL) {
                    if($qty == 0){
                        $qty = 1;
                    }
                    $item = $this->itemRepository->get($itemId);
                    $product = $this->productRepository->get($item->getSku(), false, $this->storeManager->getStore()->getId());
                    $parent_products = $this->configurableResourceModel->getParentIdsByChild($product->getId());
                    $parents_sku = $this->resourceModel->getProductsSku($parent_products);
                    $childProducts = $this->linkManagment->getChildren($parents_sku[0]['sku']);

                    foreach ($childProducts as $child) {
                        if ($this->isSaleable($child)) {
                            if($child->getId() === null){
                                $child->setId($child->getIdBySku($child->getSku()));
                            }
                            if ($this->getProductQtyById($child->getId()) >= $this->getMinQty() + $qty && $product->getId() != $child->getId()) {
                                $size[] = ["value" => $child->getId(), "text" => $child->getResource()->getAttribute('size')->getFrontend()->getValue($child), "selected" => false];
                            }
                        }
                    }
                    if (count($size) == 0) {
                        $size = array(["value" => 0, "text" => "No Available Size", "selected" => true]);
                    }
                }
                return $size;
            } catch (\Exception $e) {
                $this->logger->critical($e->getMessage());
                return [];
            }
        }else{
            return  array(["value" => 0, "text" => "The Reason Selected is not compatible with Change Size Resolution.", "selected" => true]);
        }

    }


    /**
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\Result\Json|\Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $result = $this->jsonFactory->create();
        try
        {
            $itemId = $this->getRequest()->getParam('itemId');
            $qty = $this->getRequest()->getParam('qty');
            $resolution = $this->getRequest()->getParam('resolution');



            return $result->setData($size);
        }
        catch (\Exception $e)
        {
            $size = array(["value" => 0, "text" => "No Available Size", "selected" => true]);
            return $result->setData($size);
        }
    }

    /**
     * @param $product
     * @return bool
     */
    public function isSaleable($product)
    {
        $storeId = $this->storeManager->getStore()->getId();
        $products = $this->collectionFactory->create()
            ->setStoreId($storeId)
            ->addPriceData($this->customerSession->getCustomerGroupId(),$this->storeManager->getStore()->getWebsiteId())
            ->addAttributeToFilter('sku',['eq'=>$product->getSku()])
            ->getItems();
        if($products)
        {
            return true;
        }
        else
        {
            return false;
        }

    }


    /**
     * @param $productId
     * @return float
     */
    public function getProductQtyById($productId)
    {
        return $this->stockItem->getStockQty($productId);
    }

    /**
     * @return mixed
     */
    public function getMinQty()
    {
        return (int) $this->scopeConfig->getValue(self::STOCK_MIN_QTY);
    }


}


